from . import (
    test_doc,
    test_doctests,
    test_fastobo,
    test_header,
    test_id,
    test_pv,
    test_qualifier,
    test_term,
    test_typedef,
    test_xref
)

def load_tests(loader, suite, pattern):
    suite.addTests(loader.loadTestsFromModule(test_doc))
    suite.addTests(loader.loadTestsFromModule(test_doctests))
    suite.addTests(loader.loadTestsFromModule(test_fastobo))
    suite.addTests(loader.loadTestsFromModule(test_header))
    suite.addTests(loader.loadTestsFromModule(test_id))
    suite.addTests(loader.loadTestsFromModule(test_pv))
    suite.addTests(loader.loadTestsFromModule(test_qualifier))
    suite.addTests(loader.loadTestsFromModule(test_term))
    suite.addTests(loader.loadTestsFromModule(test_typedef))
    suite.addTests(loader.loadTestsFromModule(test_xref))
    return suite



# coding: utf-8

import datetime
import unittest

import fastobo


# --- TermFrame --------------------------------------------------------------

class _TestFrame(object):

    Frame = NotImplementedError
    NameClause = NotImplementedError
    CreatedByClause = NotImplementedError

    def setUp(self):
        self.id = fastobo.id.PrefixedIdent("MS", "1000031")

    def test_init(self):
        try:
            frame = self.Frame(self.id)
        except Exception:
            self.fail("could not create frame instances")

    def test_init_iterable(self):
        try:
            frame = self.Frame(self.id, [])
        except Exception:
            self.fail("could not create frame instances")
        try:
            frame = self.Frame(self.id, [
                self.NameClause("thing"),
                self.CreatedByClause("Martin Larralde")
            ])
        except Exception:
            self.fail("could not create frame from iterable")

    def test_init_type_error(self):
        self.assertRaises(TypeError, self.Frame, 1)
        self.assertRaises(TypeError, self.Frame, [1])
        self.assertRaises(TypeError, self.Frame, ["abc"])
        self.assertRaises(TypeError, self.Frame, "abc")
        self.assertRaises(TypeError, self.Frame, self.id, 1)
        self.assertRaises(TypeError, self.Frame, self.id, [1])
        self.assertRaises(TypeError, self.Frame, self.id, ["abc"])
        self.assertRaises(TypeError, self.Frame, self.id, "abc")

    def test_append(self):
        frame = self.Frame(self.id)
        self.assertEqual(len(frame), 0)
        c1 = self.NameClause("thing")
        frame.append(c1)
        self.assertEqual(len(frame), 1)
        self.assertEqual(frame[0], c1)
        c2 = self.CreatedByClause("Martin Larralde")
        frame.append(c2)
        self.assertEqual(len(frame), 2)
        self.assertEqual(frame[0], c1)
        self.assertEqual(frame[1], c2)

    def test_reverse(self):
        c1 = self.NameClause("thing")
        c2 = self.CreatedByClause("Martin Larralde")
        frame = self.Frame(self.id, [c1, c2])
        self.assertEqual(list(frame), [c1, c2])
        frame.reverse()
        self.assertEqual(list(frame), [c2, c1])

    def test_clear(self):
        c1 = self.NameClause("thing")
        c2 = self.CreatedByClause("Martin Larralde")
        frame = self.Frame(self.id, [c1, c2])
        self.assertEqual(len(frame), 2)
        frame.clear()
        self.assertEqual(len(frame), 0)
        self.assertEqual(list(frame), [])

    def test_pop(self):
        c1 = self.NameClause("thing")
        c2 = self.CreatedByClause("Martin Larralde")
        frame = self.Frame(self.id, [c1, c2])
        self.assertEqual(len(frame), 2)
        x1 = frame.pop()
        self.assertEqual(len(frame), 1)
        self.assertEqual(x1, c2)
        x2 = frame.pop()
        self.assertEqual(len(frame), 0)
        self.assertEqual(x2, c1)
        self.assertRaises(IndexError, frame.pop)

# --- DefClause --------------------------------------------------------------

class _TestDefClause(object):

    type = NotImplementedError

    def test_repr(self):
        clause = self.type("definition")
        self.assertEqual(repr(clause), "DefClause('definition')")

        id_ = fastobo.id.PrefixedIdent('ISBN', '0321842685')
        desc = "Hacker's Delight (2nd Edition)"
        x = fastobo.xref.Xref(id_, desc)

        clause = self.type("definition", fastobo.xref.XrefList([x]))
        self.assertEqual(repr(clause), "DefClause('definition', XrefList([{!r}]))".format(x))


# --- ConsiderClause ---------------------------------------------------------

class _TestConsiderClause(object):

    type = NotImplementedError

    def setUp(self):
        self.id = fastobo.id.PrefixedIdent("MS", "1000031")
        self.id2 = fastobo.id.PrefixedIdent("MS", "1000032")

    def test_init(self):
        try:
            frame = self.type(self.id)
        except Exception:
            self.fail("could not create `ConsiderClause` instances")

    def test_init_type_error(self):
        self.assertRaises(TypeError, self.type)
        self.assertRaises(TypeError, self.type, 1)

    def test_eq(self):
        self.assertEqual(self.type(self.id), self.type(self.id))
        self.assertNotEqual(self.type(self.id), self.type(self.id2))


# --- IsObsoleteClause -------------------------------------------------------

class _TestIsObsoleteClause(object):

    type = NotImplementedError

    def test_init(self):
        try:
            frame = self.type(True)
        except Exception:
            self.fail("could not create `IsObsoleteClause` instances")

    def test_property_obsolete(self):
        c = self.type(False)
        self.assertEqual(c.obsolete, False)
        c.obsolete = True
        self.assertEqual(c.obsolete, True)

    def test_repr(self):
        self.assertEqual(repr(self.type(False)), "IsObsoleteClause(False)")
        self.assertEqual(repr(self.type(True)), "IsObsoleteClause(True)")

    def test_str(self):
        self.assertEqual(str(self.type(False)), "is_obsolete: false")
        self.assertEqual(str(self.type(True)), "is_obsolete: true")

    def test_eq(self):
        self.assertEqual(self.type(True), self.type(True))
        self.assertEqual(self.type(False), self.type(False))
        self.assertNotEqual(self.type(False), self.type(True))


# --- CreationDateClause -----------------------------------------------------

class _TestCreationDateClause(object):

    type = NotImplementedError

    def test_date(self):
        d1 = datetime.date(2021, 1, 23)
        clause = self.type(d1)
        self.assertEqual(str(clause), "creation_date: 2021-01-23")
        self.assertEqual(repr(clause), "CreationDateClause(datetime.date(2021, 1, 23))")
        self.assertEqual(clause.date, d1)
        self.assertIsInstance(clause.date, datetime.date)
        d2 = datetime.date(2021, 2, 15)
        clause.date = d2
        self.assertIsInstance(clause.date, datetime.date)

    def test_datetime(self):
        d1 = datetime.datetime(2021, 1, 23, 12)
        clause = self.type(d1)
        self.assertEqual(str(clause), "creation_date: 2021-01-23T12:00:00")
        self.assertEqual(repr(clause), "CreationDateClause(datetime.datetime(2021, 1, 23, 12, 0))")
        self.assertEqual(clause.date, d1)
        self.assertIsInstance(clause.date, datetime.datetime)
        d2 = datetime.datetime(2021, 2, 15, 12, 30, 0, tzinfo=datetime.timezone.utc)
        clause.date = d2
        self.assertEqual(str(clause), "creation_date: 2021-02-15T12:30:00Z")
        self.assertIsInstance(clause.date, datetime.datetime)



# coding: utf-8

import datetime
import unittest

import fastobo

from .common import (
    _TestFrame,
    _TestIsObsoleteClause,
    _TestDefClause,
    _TestConsiderClause,
    _TestCreationDateClause,
)

# --- TypedefFrame -----------------------------------------------------------

class TestTypedefFrame(_TestFrame, unittest.TestCase):
    Frame = fastobo.typedef.TypedefFrame
    NameClause = fastobo.typedef.NameClause
    CreatedByClause = fastobo.typedef.CreatedByClause


# --- DefClause --------------------------------------------------------------

class TestDefClause(_TestDefClause, unittest.TestCase):
    type = fastobo.typedef.DefClause


# --- ConsiderClause ---------------------------------------------------------

class TestConsiderClause(_TestConsiderClause, unittest.TestCase):
    type = fastobo.typedef.ConsiderClause


# --- IsObsoleteClause -------------------------------------------------------

class TestIsObsoleteClause(_TestIsObsoleteClause, unittest.TestCase):
    type = fastobo.typedef.IsObsoleteClause


# --- CreationDateClause -----------------------------------------------------

class TestCreationDateClause(_TestCreationDateClause, unittest.TestCase):
    type = fastobo.typedef.CreationDateClause



# coding: utf-8

import datetime
import unittest

import fastobo

from .common import (
    _TestFrame,
    _TestIsObsoleteClause,
    _TestDefClause,
    _TestConsiderClause,
    _TestCreationDateClause,
)

# --- TermFrame --------------------------------------------------------------

class TestTermFrame(_TestFrame, unittest.TestCase):
    Frame = fastobo.term.TermFrame
    NameClause = fastobo.term.NameClause
    CreatedByClause = fastobo.term.CreatedByClause


# --- DefClause --------------------------------------------------------------

class TestDefClause(_TestDefClause, unittest.TestCase):
    type = fastobo.term.DefClause


# --- ConsiderClause ---------------------------------------------------------

class TestConsiderClause(_TestConsiderClause, unittest.TestCase):
    type = fastobo.term.ConsiderClause


# --- IsObsoleteClause -------------------------------------------------------

class TestIsObsoleteClause(_TestIsObsoleteClause, unittest.TestCase):
    type = fastobo.term.IsObsoleteClause


# --- CreationDateClause -----------------------------------------------------

class TestCreationDateClause(_TestCreationDateClause, unittest.TestCase):
    type = fastobo.term.CreationDateClause



# coding: utf-8

import io
import os
import unittest
import pathlib

import fastobo

MS = os.path.realpath(os.path.join(__file__, "..", "data", "ms.obo"))
MS_FRAMES = 2941

class TestLoad(unittest.TestCase):

    def test_file_not_found(self):
        self.assertRaises(FileNotFoundError, fastobo.load, "abcdef")

    def test_type_error(self):
        self.assertRaises(TypeError, fastobo.load, 1)
        self.assertRaises(TypeError, fastobo.load, [])

        with open(MS) as f:
            self.assertRaises(TypeError, fastobo.load, f)

    def test_error_propagation(self):

        def read(x):
            if x == 0:
                return b''
            raise RuntimeError(x)

        with open(MS, 'rb') as f:
            f.read = read
            self.assertRaises(RuntimeError, fastobo.load, f)

    def test_syntax_error(self):
        self.assertRaises(SyntaxError, fastobo.loads, "hello there")

    def test_threading_single(self):
        doc = fastobo.load(MS, threads=1)
        self.assertEqual(len(doc), MS_FRAMES)

        with open(MS, 'rb') as f:
            doc = fastobo.load(f, threads=1)
            self.assertEqual(len(doc), MS_FRAMES)

    def test_threading_explicit(self):
        doc = fastobo.load(MS, threads=4)
        self.assertEqual(len(doc), MS_FRAMES)

        with open(MS, 'rb') as f:
            doc = fastobo.load(f, threads=4)
            self.assertEqual(len(doc), MS_FRAMES)

    def test_threading_detect(self):
        doc = fastobo.load(MS, threads=0)
        self.assertEqual(len(doc), MS_FRAMES)

        with open(MS, 'rb') as f:
            doc = fastobo.load(f, threads=0)
            self.assertEqual(len(doc), MS_FRAMES)

    def test_threading_invalid(self):
        self.assertRaises(ValueError, fastobo.load, MS, threads=-1)

    def test_load_path(self):
        doc = fastobo.load(pathlib.Path(MS))
        self.assertEqual(len(doc), MS_FRAMES)


class TestIter(unittest.TestCase):

    def test_file_not_found(self):
        self.assertRaises(FileNotFoundError, fastobo.iter, "abcdef")

    def test_type_error(self):
        self.assertRaises(TypeError, fastobo.iter, 1)
        self.assertRaises(TypeError, fastobo.iter, [])

        with open(MS) as f:
            self.assertRaises(TypeError, fastobo.iter, f)

    def test_syntax_error(self):
        f = io.BytesIO(b"format-version: 1.4\ndate: 05:20:2021 12:00\n")
        self.assertRaises(SyntaxError, fastobo.iter, f)

    def test_threading_single(self):
        frame_count = sum(1 for _ in fastobo.iter(MS, threads=1))
        self.assertEqual(frame_count, MS_FRAMES)

        with open(MS, 'rb') as f:
            frame_count = sum(1 for _ in fastobo.iter(f, threads=1))
            self.assertEqual(frame_count, MS_FRAMES)

    def test_threading_explicit(self):
        frame_count = sum(1 for _ in fastobo.iter(MS, threads=4))
        self.assertEqual(frame_count, MS_FRAMES)

        with open(MS, 'rb') as f:
            frame_count = sum(1 for _ in fastobo.iter(f, threads=4))
            self.assertEqual(frame_count, MS_FRAMES)

    def test_threading_detect(self):
        frame_count = sum(1 for _ in fastobo.iter(MS, threads=0))
        self.assertEqual(frame_count, MS_FRAMES)

        with open(MS, 'rb') as f:
            frame_count = sum(1 for _ in fastobo.iter(f, threads=0))
            self.assertEqual(frame_count, MS_FRAMES)

    def test_threading_invalid(self):
        self.assertRaises(ValueError, fastobo.iter, MS, threads=-1)

    def test_iter_path(self):
        n = sum(1 for _ in fastobo.iter(pathlib.Path(MS)))
        self.assertEqual(n, MS_FRAMES)


class TestLoads(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        with open(MS, 'r') as f:
            cls.text = f.read()

    def test_threading_single(self):
        doc = fastobo.loads(self.text, threads=1)
        self.assertEqual(len(doc), MS_FRAMES)

    def test_threading_explicit(self):
        doc = fastobo.loads(self.text, threads=4)
        self.assertEqual(len(doc), MS_FRAMES)

    def test_threading_detect(self):
        doc = fastobo.loads(self.text, threads=0)
        self.assertEqual(len(doc), MS_FRAMES)

    def test_threading_invalid(self):
        self.assertRaises(ValueError, fastobo.loads, self.text, threads=-1)



# coding: utf-8

import datetime
import unittest

import fastobo


class TestQualifier(unittest.TestCase):

    type = fastobo.qual.Qualifier

    def test_init(self):
        key = fastobo.id.UnprefixedIdent("source")
        value = "ISBN:0321842685"
        try:
            qual = self.type(key, value)
        except Exception:
            self.fail("could not create `Qualifier` instance")

    def test_init_type_error(self):
        key = fastobo.id.UnprefixedIdent("source")
        value = "ISBN:0321842685"
        self.assertRaises(TypeError, self.type, 1)
        self.assertRaises(TypeError, self.type, 1, value)
        self.assertRaises(TypeError, self.type, key, 1)

    def test_str(self):
        key = fastobo.id.UnprefixedIdent("source")
        value = "ISBN:0321842685"
        self.assertEqual(str(self.type(key, value)), 'source="ISBN:0321842685"')

    def test_eq(self):
        i1 = fastobo.id.UnprefixedIdent('a')
        i2 = fastobo.id.UnprefixedIdent('b')
        q1 = self.type(i1, "test")
        self.assertEqual(q1, q1)
        q2 = self.type(i1, "test")
        self.assertIsNot(q1, q2)
        self.assertEqual(q1, q2)
        q3 = self.type(i2, "test")
        self.assertNotEqual(q1, q3)
        q4 = self.type(i1, "test2")
        self.assertNotEqual(q1, q4)


class TestQualifierList(unittest.TestCase):

    type = fastobo.qual.QualifierList

    def setUp(self):
        self.q1 = fastobo.qual.Qualifier(fastobo.id.UnprefixedIdent('source'), "ISBN:0321842685")
        self.q2 = fastobo.qual.Qualifier(fastobo.id.UnprefixedIdent("minCardinality"), "2")

    def test_init(self):
        try:
            ql = self.type()
        except Exception:
            self.fail("could not create `QualifierList` instance without argument")
        try:
            ql = self.type([self.q1, self.q2])
        except Exception:
            self.fail("could not create `QualifierList` instance from list")
        try:
            ql = self.type(iter([self.q1, self.q2]))
        except Exception:
            self.fail("could not create `QualifierList` instance from iterator")

    def test_init_type_error(self):
        # Errors on an iterator of type != Qualifier
        self.assertRaises(TypeError, self.type, "abc")
        self.assertRaises(TypeError, self.type, ["abc", "def"])

    def test_str(self):
        q1, q2 = self.q1, self.q2
        self.assertEqual(str(self.type()), "{}")
        self.assertEqual(str(self.type([q1])), '{source="ISBN:0321842685"}')
        self.assertEqual(str(self.type([q1, q2])), '{source="ISBN:0321842685", minCardinality="2"}')

    def test_append(self):
        q1, q2 = self.q1, self.q2
        l = self.type()
        self.assertEqual(len(l), 0)
        l.append(q1)
        self.assertEqual(len(l), 1)
        self.assertEqual(l[0], q1)
        l.append(q2)
        self.assertEqual(len(l), 2)
        self.assertEqual(l[0], q1)
        self.assertEqual(l[1], q2)

    def test_contains(self):
        q1, q2 = self.q1, self.q2
        l1 = self.type()
        self.assertNotIn(q1, l1)
        self.assertNotIn(q2, l1)
        l2 = self.type([q1])
        self.assertIn(q1, l2)
        self.assertNotIn(q2, l2)
        l3 = self.type([q1, q2])
        self.assertIn(q1, l3)
        self.assertIn(q2, l3)

    def test_repr(self):
        q1, q2 = self.q1, self.q2
        self.assertEqual( repr(self.type()), "QualifierList()" )
        self.assertEqual( repr(self.type([q1])), "QualifierList([{!r}])".format(q1) )
        self.assertEqual( repr(self.type([q1, q2])), "QualifierList([{!r}, {!r}])".format(q1, q2) )
